%-------------------------------------------------------------------------------------------------------------------
%------------------Introduction to Price-Managed Heavy Good Electric Vehicle (HGEV) Depot Charging------------------
%-------------------------------------------Demand Modeling Software------------------------------------------------
%-------------------------------------------------------------------------------------------------------------------
% Introducing the "Price-Managed Heavy Good Electric Vehicle (HGEV) Charging Modelling": 
% This software package is designed to model the charging demand profile of a fleet of Heavy Goods Electric Vehicles (HGEV) 
% operating in a depot style while managing the charging slots to minimize the total price considering the electricity price 
% variation in the region of operation (UK for the current setting).
% This is the main controlling part of minimised cost depot vehicle charge modeling, it accepts depot and vehicle specifications, 
% and calls the relevant functions to complete the modeling of the load profile.
%**************************************************************************
clc; clear;close all;
%%*************************************************************************
%--------------------------------------------------------------------------
%--------------------------Definition of Input Data------------------------
%--------------------------------------------------------------------------
%%*************************Fleet Specifications****************************
FleetSize=50;                                    % number of the vehicles (change for different fleet size) 
BatteryCapacity = 400;                           % battery capacity [kWh]
SpecificConsumption   = 1.8;                     % specific energy consumption[kWh/Mile]
%%*************************Depot Specifications****************************
ChargingPower = 50;                               % power rating of a charger[kW]
DepotFactor=0.7;                                  % depot limiting factor (is used to set a cap for the total amount of power(depot limit) available in the depot, the maximum is "1")
                                                  % Note: for FleetSize = 1; DepotFactor must be equal to 1
%%***************************Charging Sart Time****************************
StartHour=19;                                     % input hours (24 hour format)
StartMinute=0;                                    % input minutes
StartSecond=0;                                    % input seconds
%%*************************************************************************
%--------------------------------------------------------------------------
%---------------The definition of Input Data has been finalised.-----------
%-----------------It is time to proceed with running the model.------------
%--------------------------------------------------------------------------
%**************************************************************************
DepotLimit= floor((FleetSize*DepotFactor))*ChargingPower;     % Power Limit of depot 
%**************************************************************************
FileName = sprintf('FleetLoadProfile_%s.xlsx', datestr(now,'yyyy-mm-dd_HH-MM')); % The file where results are saved
%%*********Calls the fuction to get fleet traveled distanaces(at the end of daily operation)***
FleetTravel= FuncTraveledDistance(FleetSize);
%%*********Get SOC of the fleet(at the end of daily operation)
[FleetStateOfCharge, FleetRequiredEnergy] = FuncFleetStateOfCharge(BatteryCapacity,SpecificConsumption,FleetTravel); 
writematrix('Vehicle',FileName,'Sheet',3,'Range','A1'); % Writes the header for the data on the relevent column
writematrix((1:FleetSize)',FileName,'Sheet',3,'Range','A2'); % Writes the header for the data on the relevent column
writematrix('State of Charge',FileName,'Sheet',3,'Range','B1'); % Writes the header for the data on the relevent column
writematrix((FleetStateOfCharge'),FileName,'Sheet',3,'Range','B2'); % Writes the header for the data on the relevent column
writematrix('Required Energy',FileName,'Sheet',3,'Range','C1');
writematrix((FleetRequiredEnergy'),FileName,'Sheet',3,'Range','C2');
%**************************************************************************
%%*********Find Minimized Time Slots (multiplied by 2 because half-hour price available) 
StartTimeinHours=StartHour+StartMinute/60+StartSecond/3600;
PriceBasedChargeStartingTime=floor(StartTimeinHours);        % the starting time (hour) slot of current day 
DayStartTime=2*PriceBasedChargeStartingTime+1;
PriceBasedChargeEndinTime=6;                             % the ending time (hour) slot of next day
DayEndTime=2*(24+PriceBasedChargeEndinTime); 
%%************************Charging time setting ***************************
StartTimeSetting= (StartTimeinHours)*3600;               % time step : one second
[MinimisedTimeSlots, PowerAvailablility]=FuncPriceMinimizing(FleetSize,FleetRequiredEnergy,ChargingPower,DepotLimit,DayStartTime,DayEndTime); %FuncCostMinimizing20210925
%%*********Storing the minimized time slots
FuncStoreMinPriceTimeSlots(FileName, FleetSize, MinimisedTimeSlots, PowerAvailablility) ;
% **********************Call Vehcile Charging Function*********************
TotalPower=0;
%%**********Alocating the first set of vehicles to be charged**************
FirstFleetSet=floor(DepotLimit/ChargingPower);  % calculates the first set of vehicles to be charged
RemainingFleet=FleetSize-FirstFleetSet;
SoCmaxTimeRecorederFirstSet=[];
for Vehicle=1:FirstFleetSet   
    VehicleName = strcat( 'Vehicle_',num2str(Vehicle) );
    Alphabet = reshape('A':'Z',[],1);
    Column = num2cell(Alphabet(1+rem(Vehicle-1,26)));
    v = fix((Vehicle-1)/26);
    while any(v), Column(v>0) = strcat(Alphabet(1+rem(v(v>0)-1,26)),Column(v>0)); v = fix((v-1)/26); end
    Column=Column{1}    
    writematrix(VehicleName,FileName,'Sheet',1,'Range',[Column,num2str(1)]); % Writes the header for the data on the relevent column   
    SOC=FleetStateOfCharge(Vehicle);
    [VehicleLoadXaxis VehicleLoad SoCmaxTimeRecoreder]= FuncVehicleCharging(FileName,Column,ChargingPower, SOC, BatteryCapacity, MinimisedTimeSlots{Vehicle});
    TotalPower=TotalPower+VehicleLoad;
    SoCmaxTimeRecorederFirstSet=[SoCmaxTimeRecorederFirstSet SoCmaxTimeRecoreder];
end
%%*************for the minimum starting time for  the remaing set**********
RecordingTimes=SoCmaxTimeRecorederFirstSet;
StartTimeRemaining= min(SoCmaxTimeRecorederFirstSet);
SoCmaxTimeRecorederAll=SoCmaxTimeRecorederFirstSet;
for Vehicle=FirstFleetSet+1:FleetSize    
    VehicleName = strcat( 'Vehicle_',num2str(Vehicle));    
    Alphabet = reshape('A':'Z',[],1);
    Column = num2cell(Alphabet(1+rem(Vehicle-1,26)));
    v = fix((Vehicle-1)/26);
    while any(v), Column(v>0) = strcat(Alphabet(1+rem(v(v>0)-1,26)),Column(v>0)); v = fix((v-1)/26); end
    Column=Column{1}    
    writematrix(VehicleName,FileName,'Sheet',1,'Range',[Column,num2str(1)]); % Writes the header for the data on the relevent column    
    SOC=FleetStateOfCharge(Vehicle);    
    if (StartTimeRemaining >= 3600*24)
          StartTimeRem=StartTimeRemaining-24*3600+1;  
    else
          StartTimeRem=StartTimeRemaining+1;  
    end     
    SoCmaxTimeRecorederAll(SoCmaxTimeRecorederAll==min(SoCmaxTimeRecorederAll))=10e10;    
   [VehicleLoadXaxis VehicleLoad SoCmaxTimeRecoreder]= FuncVehicleCharging(FileName,Column,ChargingPower, SOC,BatteryCapacity, MinimisedTimeSlots{Vehicle});
    TotalPower=TotalPower+VehicleLoad;
    %********************finding minimum Starting time ********************   
    SoCmaxTimeRecorederAll=[SoCmaxTimeRecorederAll SoCmaxTimeRecoreder];
    StartTimeRemaining= min(SoCmaxTimeRecorederAll);
end
writematrix('Demand',FileName,'Sheet',2,'Range','A1'); % Writes the header for the data on the relevent column
writematrix((TotalPower),FileName,'Sheet',2,'Range','A2'); % Writes the header for the data on the relevent column
%%********calls to plot the total demand of fleet in 48 hour format********
 FuncReadTotalDemand(FileName,VehicleLoadXaxis); 
%%********calls to plot the total demand in a single 24-hour format********
 FuncPlottingInSingle24hourFormat(FileName);
%%*************************************************************************